@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2003-2007 Imendio AB
  * Copyright (C) 2007-2011 Collabora Ltd.
@@ -39,7 +38,8 @@
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/defs.h>
 #include <telepathy-glib/util.h>
-
+#include <telepathy-logger/call-event.h>
+#include <telepathy-logger/call-event-internal.h>
 #include <telepathy-logger/event-internal.h>
 #include <telepathy-logger/text-event.h>
 #include <telepathy-logger/text-event-internal.h>
@@ -66,6 +66,10 @@
 #define LOG_FOOTER \
     "</log>\n"
 
+#define ALL_SUPPORTED_TYPES (TPL_EVENT_MASK_TEXT | TPL_EVENT_MASK_CALL)
+#define CONTAINS_ALL_SUPPORTED_TYPES(type_mask) \
+  (((type_mask) & ALL_SUPPORTED_TYPES) == ALL_SUPPORTED_TYPES)
+
 
 struct _TplLogStoreXmlPriv
 {
@@ -501,23 +505,21 @@ add_text_event (TplLogStoreXml *self,
 
   sender = tpl_event_get_sender (TPL_EVENT (message));
   contact_id = g_markup_escape_text (tpl_entity_get_identifier (sender), -1);
-  if (tpl_entity_get_alias (sender) != NULL)
-    contact_name = g_markup_escape_text (tpl_entity_get_alias (sender), -1);
-  if (tpl_entity_get_avatar_token (sender) != NULL)
-    avatar_token = g_markup_escape_text (tpl_entity_get_avatar_token
-        (sender), -1);
+  contact_name = g_markup_escape_text (tpl_entity_get_alias (sender), -1);
+  avatar_token = g_markup_escape_text (tpl_entity_get_avatar_token (sender),
+      -1);
 
   event = g_strdup_printf ("<message time='%s' id='%s' name='%s' "
       "token='%s' isuser='%s' type='%s'>"
       "%s</message>\n" LOG_FOOTER, timestamp,
       contact_id, contact_name,
-      avatar_token ? avatar_token : "",
+      avatar_token,
       tpl_entity_get_entity_type (sender)
           == TPL_ENTITY_SELF ? "true" : "false",
       _tpl_text_event_message_type_to_str (msg_type),
       body);
 
-  DEBUG ("writing event from %s (ts %s)",
+  DEBUG ("writing text event from %s (ts %s)",
       contact_id, timestamp);
 
   ret = _log_store_xml_write_to_store (self, account,
@@ -537,6 +539,100 @@ out:
   return ret;
 }
 
+static gboolean
+add_call_event (TplLogStoreXml *self,
+    TplCallEvent *event,
+    GError **error)
+{
+  gboolean ret = FALSE;
+  TpDBusDaemon *bus_daemon;
+  TpAccount *account;
+  TplEntity *sender;
+  TplEntity *actor;
+  TplEntity *target;
+  gchar *timestamp;
+  gchar *sender_avatar = NULL;
+  gchar *sender_name = NULL;
+  gchar *sender_id;
+  gchar *actor_name = NULL;
+  gchar *actor_avatar = NULL;
+  gchar *actor_id;
+  gchar *log_str;
+  TplCallEndReason reason;
+
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+  g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), FALSE);
+  g_return_val_if_fail (TPL_IS_CALL_EVENT (event), FALSE);
+
+  bus_daemon = tp_dbus_daemon_dup (error);
+  if (bus_daemon == NULL)
+    {
+      DEBUG ("Error acquiring bus daemon: %s", (*error)->message);
+      goto out;
+    }
+
+  account = tpl_event_get_account (TPL_EVENT (event));
+
+  timestamp = log_store_xml_get_timestamp_from_event (
+      TPL_EVENT (event));
+  reason = tpl_call_event_get_end_reason (event);
+
+  sender = tpl_event_get_sender (TPL_EVENT (event));
+  actor = tpl_call_event_get_end_actor (event);
+  target = _tpl_event_get_target (TPL_EVENT (event));
+  sender_id = g_markup_escape_text (tpl_entity_get_identifier (sender), -1);
+  sender_name = g_markup_escape_text (tpl_entity_get_alias (sender), -1);
+  sender_avatar = g_markup_escape_text (tpl_entity_get_avatar_token (sender),
+      -1);
+  actor_id = g_markup_escape_text (tpl_entity_get_identifier (actor), -1);
+  actor_name = g_markup_escape_text (tpl_entity_get_alias (actor), -1);
+  actor_avatar = g_markup_escape_text (tpl_entity_get_avatar_token (actor),
+      -1);
+
+
+  log_str = g_strdup_printf ("<call time='%s' "
+      "id='%s' name='%s' isuser='%s' token='%s' "
+      "duration='%" G_GINT64_FORMAT "' "
+      "actor='%s' actortype='%s' "
+      "actorname='%s' actortoken='%s' "
+      "reason='%s' detail='%s'/>\n"
+      LOG_FOOTER,
+        timestamp,
+        sender_id,
+        sender_name,
+        tpl_entity_get_entity_type (sender) ==
+            TPL_ENTITY_SELF ? "true" : "false",
+        sender_avatar,
+        tpl_call_event_get_duration (event),
+        actor_id,
+        _tpl_entity_type_to_str (tpl_entity_get_entity_type (actor)),
+        actor_name,
+        actor_avatar,
+        _tpl_call_event_end_reason_to_str (reason),
+        tpl_call_event_get_detailed_end_reason (event));
+
+  DEBUG ("writing call event from %s (ts %s)",
+      tpl_entity_get_identifier (target),
+      timestamp);
+
+  ret = _log_store_xml_write_to_store (self, account, target, log_str, error);
+
+out:
+  g_free (sender_id);
+  g_free (sender_name);
+  g_free (sender_avatar);
+  g_free (actor_id);
+  g_free (actor_name);
+  g_free (actor_avatar);
+  g_free (timestamp);
+  g_free (log_str);
+
+  if (bus_daemon != NULL)
+    g_object_unref (bus_daemon);
+
+  return ret;
+}
+
 
 /* First of two phases selection: understand the type Event */
 static gboolean
@@ -551,6 +647,8 @@ log_store_xml_add_event (TplLogStore *store,
 
   if (TPL_IS_TEXT_EVENT (event))
     return add_text_event (self, TPL_TEXT_EVENT (event), error);
+  else if (TPL_IS_CALL_EVENT (event))
+    return add_call_event (self, TPL_CALL_EVENT (event), error);
 
   DEBUG ("TplEntry not handled by this LogStore (%s). "
       "Ignoring Event", log_store_xml_get_name (store));
@@ -574,8 +672,9 @@ log_store_xml_exists (TplLogStore *store,
   g_return_val_if_fail (TP_IS_ACCOUNT (account), FALSE);
   g_return_val_if_fail (TPL_IS_ENTITY (target), FALSE);
 
-  if (!(type_mask & TPL_EVENT_MASK_TEXT))
-    return FALSE;
+  /* FIXME This method is exposed synchronously in the log manager API and
+   * thus we need a constant time reply. The implementation is not 100%
+   * correct here, but provide this constant time. See fd.o but #35549. */
 
   dir = log_store_xml_get_dir (self, account, target);
   exists = g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
@@ -606,6 +705,41 @@ create_date_from_string (const gchar *str)
   return date;
 }
 
+
+static gboolean
+log_store_xml_match_in_file (const gchar *filename,
+    GRegex *regex)
+{
+  gboolean retval = FALSE;
+  GMappedFile *file;
+  gsize length;
+  gchar *contents = NULL;
+
+  file = g_mapped_file_new (filename, FALSE, NULL);
+  if (file == NULL)
+    goto out;
+
+  length = g_mapped_file_get_length (file);
+  contents = g_mapped_file_get_contents (file);
+
+  if (length == 0 || contents == NULL)
+    goto out;
+
+  retval = g_regex_match_full (regex, contents, length, 0, 0, NULL, NULL);
+
+  DEBUG ("%s pattern '%s' in file '%s'",
+      retval ? "Matched" : "Not matched",
+      g_regex_get_pattern (regex),
+      filename);
+
+out:
+  if (file != NULL)
+    g_mapped_file_unref (file);
+
+  return retval;
+}
+
+
 static GList *
 log_store_xml_get_dates (TplLogStore *store,
     TpAccount *account,
@@ -614,53 +748,92 @@ log_store_xml_get_dates (TplLogStore *store,
 {
   TplLogStoreXml *self = (TplLogStoreXml *) store;
   GList *dates = NULL;
-  gchar *directory;
-  GDir *dir;
-  const gchar *filename;
-  const gchar *p;
+  gchar *directory = NULL;
+  GDir *dir = NULL;
+  GString *pattern = NULL;
+  GRegex *regex = NULL;
+  GError *error = NULL;
+  const gchar *basename;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), NULL);
   g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
 
-  if (!(type_mask & TPL_EVENT_MASK_TEXT))
-    return NULL;
-
   directory = log_store_xml_get_dir (self, account, target);
   dir = g_dir_open (directory, 0, NULL);
   if (!dir)
     {
       DEBUG ("Could not open directory:'%s'", directory);
-      g_free (directory);
-      return NULL;
+      goto out;
     }
 
   DEBUG ("Collating a list of dates in:'%s'", directory);
 
-  while ((filename = g_dir_read_name (dir)) != NULL)
+  pattern = g_string_new ("");
+
+  if (type_mask & TPL_EVENT_MASK_TEXT)
+    g_string_append (pattern, "<message ");
+
+  if (type_mask & TPL_EVENT_MASK_CALL)
+    g_string_append_printf (pattern,
+        "%s<call ",
+        pattern->len == 0 ? "" : "|");
+
+  if (pattern->len == 0)
+    goto out;
+
+  regex = g_regex_new (pattern->str, G_REGEX_OPTIMIZE, 0, &error);
+  if (regex == NULL)
     {
-      gchar *str;
-      GDate *date;
-
-      if (!g_str_has_suffix (filename, LOG_FILENAME_SUFFIX))
-        continue;
-
-      p = strstr (filename, LOG_FILENAME_SUFFIX);
-      str = g_strndup (filename, p - filename);
-
-      if (str == NULL)
-        continue;
-
-      date = create_date_from_string (str);
-      if (date != NULL)
-       dates = g_list_insert_sorted (dates, date,
-           (GCompareFunc) g_date_compare);
-
-      g_free (str);
+      DEBUG ("Failed to create regex: %s", error->message);
+      g_error_free (error);
+      goto out;
     }
 
+  while ((basename = g_dir_read_name (dir)) != NULL)
+    {
+      gchar *filename;
+
+      if (!g_str_has_suffix (basename, LOG_FILENAME_SUFFIX))
+        continue;
+
+      filename = g_build_filename (directory, basename, NULL);
+
+      if (CONTAINS_ALL_SUPPORTED_TYPES (type_mask)
+          || log_store_xml_match_in_file (filename, regex))
+        {
+          const gchar *p;
+          gchar *str;
+          GDate *date;
+
+          p = strstr (basename, LOG_FILENAME_SUFFIX);
+          str = g_strndup (basename, p - basename);
+
+          if (str == NULL)
+            continue;
+
+          date = create_date_from_string (str);
+          if (date != NULL)
+            dates = g_list_insert_sorted (dates, date,
+                (GCompareFunc) g_date_compare);
+
+          g_free (str);
+        }
+
+      g_free (filename);
+    }
+
+out:
   g_free (directory);
-  g_dir_close (dir);
+
+  if (dir != NULL)
+    g_dir_close (dir);
+
+  if (pattern != NULL)
+    g_string_free (pattern, TRUE);
+
+  if (regex != NULL)
+    g_regex_unref (regex);
 
   DEBUG ("Parsed %d dates", g_list_length (dates));
 
@@ -765,13 +938,194 @@ log_store_xml_search_hit_new (TplLogStoreXml *self,
   return hit;
 }
 
+
+static TplEvent *
+parse_text_node (TplLogStoreXml *self,
+    xmlNodePtr node,
+    gboolean is_room,
+    const gchar *target_id,
+    const gchar *self_id,
+    TpAccount *account)
+{
+  TplEvent *event;
+  TplEntity *sender;
+  TplEntity *receiver;
+  gchar *time_str;
+  gint64 timestamp;
+  gchar *sender_id;
+  gchar *sender_name;
+  gchar *sender_avatar_token;
+  gchar *body;
+  gchar *is_user_str;
+  gboolean is_user = FALSE;
+  gchar *msg_type_str;
+  TpChannelTextMessageType msg_type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
+
+  body = (gchar *) xmlNodeGetContent (node);
+  time_str = (gchar *) xmlGetProp (node, (const xmlChar *) "time");
+  sender_id = (gchar *) xmlGetProp (node, (const xmlChar *) "id");
+  sender_name = (gchar *) xmlGetProp (node, (const xmlChar *) "name");
+  sender_avatar_token = (gchar *) xmlGetProp (node,
+      (const xmlChar *) "token");
+  is_user_str = (gchar *) xmlGetProp (node, (const xmlChar *) "isuser");
+  msg_type_str = (gchar *) xmlGetProp (node, (const xmlChar *) "type");
+
+  if (is_user_str != NULL)
+    is_user = (!tp_strdiff (is_user_str, "true"));
+
+  if (msg_type_str != NULL)
+    msg_type = _tpl_text_event_message_type_from_str (msg_type_str);
+
+  timestamp = _tpl_time_parse (time_str);
+
+  if (is_room)
+    receiver = tpl_entity_new_from_room_id (target_id);
+  else if (is_user)
+    receiver = tpl_entity_new (target_id, TPL_ENTITY_CONTACT, NULL, NULL);
+  else
+    receiver = tpl_entity_new (self_id, TPL_ENTITY_SELF,
+        tp_account_get_nickname (account), NULL);
+
+  sender = tpl_entity_new (sender_id,
+      is_user ? TPL_ENTITY_SELF : TPL_ENTITY_CONTACT,
+      sender_name, sender_avatar_token);
+
+  event = g_object_new (TPL_TYPE_TEXT_EVENT,
+      /* TplEvent */
+      "account", account,
+      /* MISSING: "channel-path", channel_path, */
+      "receiver", receiver,
+      "sender", sender,
+      "timestamp", timestamp,
+      /* TplTextEvent */
+      "message-type", msg_type,
+      "message", body,
+      NULL);
+
+  g_object_unref (sender);
+  g_object_unref (receiver);
+  xmlFree (time_str);
+  xmlFree (sender_id);
+  xmlFree (sender_name);
+  xmlFree (body);
+  xmlFree (is_user_str);
+  xmlFree (msg_type_str);
+  xmlFree (sender_avatar_token);
+
+  return event;
+}
+
+
+static TplEvent *
+parse_call_node (TplLogStoreXml *self,
+    xmlNodePtr node,
+    gboolean is_room,
+    const gchar *target_id,
+    const gchar *self_id,
+    TpAccount *account)
+{
+  TplEvent *event;
+  TplEntity *sender;
+  TplEntity *receiver;
+  TplEntity *actor;
+  gchar *time_str;
+  gint64 timestamp;
+  gchar *sender_id;
+  gchar *sender_name;
+  gchar *sender_avatar_token;
+  gchar *is_user_str;
+  gboolean is_user = FALSE;
+  gchar *actor_id;
+  gchar *actor_name;
+  gchar *actor_type;
+  gchar *actor_avatar_token;
+  gchar *duration_str;
+  gint64 duration = -1;
+  gchar *reason_str;
+  TplCallEndReason reason = TPL_CALL_END_REASON_UNKNOWN;
+  gchar *detailed_reason;
+
+  time_str = (gchar *) xmlGetProp (node, (const xmlChar *) "time");
+  sender_id = (gchar *) xmlGetProp (node, (const xmlChar *) "id");
+  sender_name = (gchar *) xmlGetProp (node, (const xmlChar *) "name");
+  sender_avatar_token = (gchar *) xmlGetProp (node,
+      (const xmlChar *) "token");
+  is_user_str = (gchar *) xmlGetProp (node, (const xmlChar *) "isuser");
+  duration_str = (char *) xmlGetProp (node, (const xmlChar*) "duration");
+  actor_id = (char *) xmlGetProp (node, (const xmlChar *) "actor");
+  actor_name = (char *) xmlGetProp (node, (const xmlChar *) "actorname");
+  actor_type = (char *) xmlGetProp (node, (const xmlChar *) "actortype");
+  actor_avatar_token = (char *) xmlGetProp (node,
+      (const xmlChar *) "actortoken");
+  reason_str = (char *) xmlGetProp (node, (const xmlChar *) "reason");
+  detailed_reason = (char *) xmlGetProp (node, (const xmlChar *) "detail");
+
+  if (is_user_str != NULL)
+    is_user = (!tp_strdiff (is_user_str, "true"));
+
+  if (reason_str != NULL)
+    reason = _tpl_call_event_str_to_end_reason (reason_str);
+
+  timestamp = _tpl_time_parse (time_str);
+
+  if (is_room)
+    receiver = tpl_entity_new_from_room_id (target_id);
+  else if (is_user)
+    receiver = tpl_entity_new (target_id, TPL_ENTITY_CONTACT, NULL, NULL);
+  else
+    receiver = tpl_entity_new (self_id, TPL_ENTITY_SELF,
+        tp_account_get_nickname (account), NULL);
+
+  sender = tpl_entity_new (sender_id,
+      is_user ? TPL_ENTITY_SELF : TPL_ENTITY_CONTACT,
+      sender_name, sender_avatar_token);
+
+  actor = tpl_entity_new (actor_id,
+      _tpl_entity_type_from_str (actor_type),
+      actor_name, actor_avatar_token);
+
+  if (duration_str != NULL)
+    duration = atoll (duration_str);
+
+  event = g_object_new (TPL_TYPE_CALL_EVENT,
+      /* TplEvent */
+      "account", account,
+      /* MISSING: "channel-path", channel_path, */
+      "receiver", receiver,
+      "sender", sender,
+      "timestamp", timestamp,
+      /* TplCallEvent */
+      "duration", duration,
+      "end-actor", actor,
+      "end-reason", reason,
+      "detailed-end-reason", detailed_reason,
+      NULL);
+
+  g_object_unref (sender);
+  g_object_unref (receiver);
+  g_object_unref (actor);
+  xmlFree (time_str);
+  xmlFree (sender_id);
+  xmlFree (sender_name);
+  xmlFree (sender_avatar_token);
+  xmlFree (is_user_str);
+  xmlFree (actor_id);
+  xmlFree (actor_name);
+  xmlFree (actor_type);
+  xmlFree (actor_avatar_token);
+  xmlFree (duration_str);
+
+  return event;
+}
+
 /* returns a Glist of TplEvent instances */
 static GList *
 log_store_xml_get_events_for_file (TplLogStoreXml *self,
     TpAccount *account,
-    const gchar *filename)
+    const gchar *filename,
+    gint type_mask)
 {
-  GList *events = NULL;
+  GQueue events = { 0 };
   xmlParserCtxtPtr ctxt;
   xmlDocPtr doc;
   xmlNodePtr log_node;
@@ -844,84 +1198,28 @@ log_store_xml_get_events_for_file (TplLogStoreXml *self,
   /* Now get the events. */
   for (node = log_node->children; node; node = node->next)
     {
-      TplTextEvent *event;
-      TplEntity *sender;
-      TplEntity *receiver;
-      gchar *time_str;
-      gint64 timestamp;
-      gchar *sender_id;
-      gchar *sender_name;
-      gchar *sender_avatar_token;
-      gchar *body;
-      gchar *is_user_str;
-      gboolean is_user = FALSE;
-      gchar *msg_type_str;
-      TpChannelTextMessageType msg_type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
+      TplEvent *event = NULL;
 
-      if (strcmp ((const gchar *) node->name, "message") != 0)
-        continue;
+      if (type_mask & TPL_EVENT_MASK_TEXT
+          && strcmp ((const gchar *) node->name, "message") == 0)
+        event = parse_text_node (self, node, is_room, target_id, self_id,
+            account);
+      else if (type_mask & TPL_EVENT_MASK_CALL
+          && strcmp ((const char*) node->name, "call") == 0)
+        event = parse_call_node (self, node, is_room, target_id, self_id,
+            account);
 
-      body = (gchar *) xmlNodeGetContent (node);
-      time_str = (gchar *) xmlGetProp (node, (const xmlChar *) "time");
-      sender_id = (gchar *) xmlGetProp (node, (const xmlChar *) "id");
-      sender_name = (gchar *) xmlGetProp (node, (const xmlChar *) "name");
-      sender_avatar_token = (gchar *) xmlGetProp (node,
-          (const xmlChar *) "token");
-      is_user_str = (gchar *) xmlGetProp (node, (const xmlChar *) "isuser");
-      msg_type_str = (gchar *) xmlGetProp (node, (const xmlChar *) "type");
-
-      if (is_user_str != NULL)
-        is_user = (!tp_strdiff (is_user_str, "true"));
-
-      if (msg_type_str != NULL)
-        msg_type = _tpl_text_event_message_type_from_str (msg_type_str);
-
-      timestamp = _tpl_time_parse (time_str);
-
-      if (is_room)
-        receiver = tpl_entity_new_from_room_id (target_id);
-      else if (is_user)
-        receiver = tpl_entity_new (target_id, TPL_ENTITY_CONTACT, NULL, NULL);
-      else
-        receiver = tpl_entity_new (self_id, TPL_ENTITY_SELF,
-            tp_account_get_nickname (account), NULL);
-
-      sender = tpl_entity_new (sender_id,
-          is_user ? TPL_ENTITY_SELF : TPL_ENTITY_CONTACT,
-          sender_name, sender_avatar_token);
-
-      event = g_object_new (TPL_TYPE_TEXT_EVENT,
-          /* TplEvent */
-          "account", account,
-          /* MISSING: "channel-path", channel_path, */
-          "receiver", receiver,
-          "sender", sender,
-          "timestamp", timestamp,
-          /* TplTextEvent */
-          "message-type", msg_type,
-          "message", body,
-          NULL);
-
-      events = g_list_append (events, event);
-
-      g_object_unref (sender);
-      g_object_unref (receiver);
-      xmlFree (time_str);
-      xmlFree (sender_id);
-      xmlFree (sender_name);
-      xmlFree (body);
-      xmlFree (is_user_str);
-      xmlFree (msg_type_str);
-      xmlFree (sender_avatar_token);
+      if (event != NULL)
+        g_queue_push_tail (&events, event);
     }
 
-  DEBUG ("Parsed %d events", g_list_length (events));
+  DEBUG ("Parsed %d events", events.length);
 
   g_free (target_id);
   xmlFreeDoc (doc);
   xmlFreeParserCtxt (ctxt);
 
-  return events;
+  return events.head;
 }
 
 
@@ -976,42 +1274,62 @@ log_store_xml_get_all_files (TplLogStoreXml *self,
 static GList *
 _log_store_xml_search_in_files (TplLogStoreXml *self,
     const gchar *text,
-    GList *files)
+    GList *files,
+    gint type_mask)
 {
   GList *l;
   GList *hits = NULL;
-  gchar *text_casefold;
+  gchar *markup_text;
   gchar *escaped_text;
+  GString *pattern = NULL;
+  GRegex *regex = NULL;
+  GError *error = NULL;
 
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (!TPL_STR_EMPTY (text), NULL);
 
-  escaped_text = g_markup_escape_text (text, -1);
-  text_casefold = g_utf8_casefold (escaped_text, -1);
+  markup_text = g_markup_escape_text (text, -1);
+  escaped_text = g_regex_escape_string (markup_text, -1);
+  g_free (markup_text);
+
+  pattern = g_string_new ("");
+
+  if (type_mask & TPL_EVENT_MASK_TEXT)
+    g_string_append_printf (pattern,
+        "<message [^>]*>[^<]*%s[^<]*</message>"
+        "|<message( [^>]* | )id='[^>]*%s[^>]*'"
+        "|<message( [^>]* | )name='[^>]*%s[^>]*'",
+        escaped_text, escaped_text, escaped_text);
+
+  if (type_mask & TPL_EVENT_MASK_CALL)
+    g_string_append_printf (pattern,
+        "%s<call( [^>]* | )id='[^>]*%s[^>]*'"
+        "|<call( [^>]* | )name='[^>]*%s[^>]*'"
+        "|<call( [^>]* | )actor='[^>]*%s[^>]*'"
+        "|<call( [^>]* | )actorname='[^>]*%s[^>]*'",
+        pattern->len == 0 ? "" : "|",
+        escaped_text, escaped_text, escaped_text, escaped_text);
+
+  if (TPL_STR_EMPTY (pattern->str))
+    goto out;
+
+  regex = g_regex_new (pattern->str,
+      G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
+      0,
+      &error);
+
+  if (!regex)
+    {
+      DEBUG ("Failed to compile regex: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
 
   for (l = files; l; l = g_list_next (l))
     {
-      gchar *filename;
-      GMappedFile *file;
-      gsize length;
-      gchar *contents = NULL;
-      gchar *contents_casefold = NULL;
+      gchar *filename = l->data;
 
-      filename = l->data;
-
-      file = g_mapped_file_new (filename, FALSE, NULL);
-      if (file == NULL)
-        goto fail;
-
-      length = g_mapped_file_get_length (file);
-      contents = g_mapped_file_get_contents (file);
-
-      if (length == 0 || contents == NULL)
-        goto fail;
-
-      contents_casefold = g_utf8_casefold (contents, length);
-
-      if (strstr (contents_casefold, text_casefold))
+      if (log_store_xml_match_in_file (filename, regex))
         {
           TplLogSearchHit *hit;
 
@@ -1024,19 +1342,18 @@ _log_store_xml_search_in_files (TplLogStoreXml *self,
                   g_date_get_month (hit->date), g_date_get_day (hit->date));
             }
         }
-
-fail:
-      if (file != NULL)
-        g_mapped_file_unref (file);
-
-      g_free (contents_casefold);
-      g_free (filename);
     }
 
-  g_list_free (files);
-  g_free (text_casefold);
+out:
   g_free (escaped_text);
 
+  if (pattern != NULL)
+    g_string_free (pattern, TRUE);
+
+  if (regex != NULL)
+    g_regex_unref (regex);
+
+  g_list_free (files);
   return hits;
 }
 
@@ -1052,13 +1369,10 @@ log_store_xml_search_new (TplLogStore *store,
   g_return_val_if_fail (TPL_IS_LOG_STORE_XML (self), NULL);
   g_return_val_if_fail (!TPL_STR_EMPTY (text), NULL);
 
-  if (!(type_mask & TPL_EVENT_MASK_TEXT))
-    return NULL;
-
   files = log_store_xml_get_all_files (self, NULL);
   DEBUG ("Found %d log files in total", g_list_length (files));
 
-  return _log_store_xml_search_in_files (self, text, files);
+  return _log_store_xml_search_in_files (self, text, files, type_mask);
 }
 
 /* Returns: (GList *) of (TplLogSearchHit *) */
@@ -1128,12 +1442,9 @@ log_store_xml_get_events_for_date (TplLogStore *store,
   g_return_val_if_fail (TPL_IS_ENTITY (target), NULL);
   g_return_val_if_fail (date != NULL, NULL);
 
-  if (!(type_mask & TPL_EVENT_MASK_TEXT))
-    return NULL;
-
   filename = log_store_xml_get_filename_for_date (self, account, target,
       date);
-  events = log_store_xml_get_events_for_file (self, account, filename);
+  events = log_store_xml_get_events_for_file (self, account, filename, type_mask);
   g_free (filename);
 
   return events;
